unit UnitSecond;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Gestures,
  FMXTee.Engine, FMXTee.Series, FMXTee.Procs, FMXTee.Chart, FMX.StdCtrls,
  FMX.Controls.Presentation, FMXTee.Series.Bubble, FMXTee.Series.World,
  FMXTee.Series.Surface, FMXTee.Series.Map;

type
  TSecondForm = class(TForm)
    ToolBar1: TToolBar;
    SpeedButton1: TSpeedButton;
    Chart1: TChart;
    GestureManager1: TGestureManager;
    SeriesMap: TWorldSeries;
    procedure Chart1Gesture(Sender: TObject; const EventInfo: TGestureEventInfo;
      var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    FLastPosition: TPointF;
    FLastDistance: Integer;
    SeriesAxis: TLineSeries;
    procedure handlePan(eventInfo: TGestureEventInfo);
    procedure handleZoom(eventInfo: TGestureEventInfo);
    procedure handleDoubletap(eventInfo: TGestureEventInfo);
  public
    { Public-Deklarationen }
    procedure ScaleMinMax;
  end;

var
  SecondForm: TSecondForm;

implementation

{$R *.fmx}

uses UnitMain, System.Math;

procedure TSecondForm.Chart1Gesture(Sender: TObject;
  const EventInfo: TGestureEventInfo; var Handled: Boolean);
begin
  if EventInfo.GestureID = igiZoom then
      handleZoom(EventInfo);
  if EventInfo.GestureID = igiPan then
     handlePan(EventInfo);
  if EventInfo.GestureID = igiDoubleTap then
     handleDoubleTap(EventInfo);

  Handled:=True;
end;

procedure TSecondForm.FormCreate(Sender: TObject);
begin
  Chart1.Zoom.Allow:=False;
  Chart1.Panning.Active:=False;
  Chart1.Title.Font.Size:=20;
  Chart1.Legend.Hide;
end;

procedure TSecondForm.FormShow(Sender: TObject);
begin
  ScaleMinMax;
end;

procedure TSecondForm.handlePan(EventInfo: TGestureEventInfo);
var
  LObj: IControl;
  chart: TChart;
  minimum,maximum,width,height: double;
  deltax,deltay:double;
begin
  LObj := Self.ObjectAtPoint(ClientToScreen(EventInfo.Location));
  if LObj is TChart then
  begin
    if not(TInteractiveGestureFlag.gfBegin in EventInfo.Flags) then
    begin
      chart:=TChart(LObj.GetObject);
      chart.Axes.Left.Automatic:=False;
      chart.Axes.Bottom.Automatic:=False;
      // Set the X coordinate.
      minimum:=chart.Axes.Bottom.Minimum;
      maximum:=chart.Axes.Bottom.Maximum;
      Width:=chart.Axes.Bottom.CalcXPosValue(maximum)-chart.Axes.Bottom.CalcXPosValue(minimum);
      deltax:=(EventInfo.Location.X-FLastPosition.X)*(maximum-minimum)/Width;
      if deltax<>0 then
         chart.Axes.Bottom.SetMinMax(minimum-deltax,maximum-deltax);
      // Set the Y coordinate.
      minimum:=chart.Axes.Left.Minimum;
      maximum:=chart.Axes.Left.Maximum;
      Height:=chart.Axes.Left.CalcYPosValue(maximum)-chart.Axes.Left.CalcYPosValue(minimum);
      deltay:=(EventInfo.Location.Y-FLastPosition.Y)*(maximum-minimum)/Height;
      if deltay<>0 then
         chart.Axes.Left.SetMinMax(minimum-deltay,maximum-deltay);
    end;

    FLastPosition := EventInfo.Location;
  end;
end;

procedure TSecondForm.handleZoom(EventInfo: TGestureEventInfo);
var
  LObj: IControl;
  chart: TChart;
  minimum,maximum: double;
  deltax,deltay,faktor:double;
begin
  LObj := Self.ObjectAtPoint(ClientToScreen(EventInfo.Location));
  if LObj is TChart then
  begin
    if not(TInteractiveGestureFlag.gfBegin in EventInfo.Flags) and
       not(TInteractiveGestureFlag.gfEnd in EventInfo.Flags) then
    begin
      chart := TChart(LObj.GetObject);
      chart.Axes.Left.Automatic:=False;
      chart.Axes.Bottom.Automatic:=False;
      // Set the X coordinate.
      minimum:=chart.Axes.Bottom.Minimum;
      maximum:=chart.Axes.Bottom.Maximum;
      if EventInfo.Distance<FLastDistance then
         Faktor:=-1
      else
         Faktor:=1;
      deltax:=0;
      if abs(maximum-minimum)<1000 then
         deltax:=faktor*10;
      if abs(maximum-minimum)<100 then
         deltax:=faktor;
      if deltax<>0 then
      begin
        minimum:=minimum+deltax;
        maximum:=maximum-deltax;
        if abs(maximum-minimum)<1000*(SeriesMap.MaxXValue-SeriesMap.MinXValue) then
        begin
          if minimum>=maximum then
             maximum:=minimum+1;
          chart.Axes.Bottom.SetMinMax(minimum,maximum);
        end;
      end;
      // Set the Y coordinate.
      minimum:=chart.Axes.Left.Minimum;
      maximum:=chart.Axes.Left.Maximum;
      if EventInfo.Distance<FLastDistance then
         Faktor:=-1
      else
         Faktor:=1;
      deltay:=0;
      if abs(maximum-minimum)<1000 then
         deltay:=faktor*10;
      if abs(maximum-minimum)<100 then
         deltay:=faktor;
      if deltay<>0 then
      begin
        minimum:=minimum+deltay;
        maximum:=maximum-deltay;
        if abs(maximum-minimum)<1000*(SeriesMap.MaxYValue-SeriesMap.MinYValue) then
        begin
          if minimum>=maximum then
             maximum:=minimum+1;
          chart.Axes.Left.SetMinMax(minimum,maximum);
        end;
      end;
    end;

    FLastDistance := EventInfo.Distance;
  end;
end;

procedure TSecondForm.handleDoubleTap(eventInfo: TGestureEventInfo);
var
  LObj: IControl;
  chart: TChart;
begin
  LObj := Self.ObjectAtPoint(ClientToScreen(EventInfo.Location));
  if LObj is TChart then
  begin
    chart:=(TChart(LObj.GetObject));
    if chart=Chart1 then
    begin
      ScaleMinMax;
    end;
  end;
end;

procedure TSecondForm.ScaleMinMax;
var
  MinX,MaxX,MinY,MaxY,DeltaX,DeltaY: double;
begin
  MinX:=SeriesMap.MinXValue;
  MaxX:=SeriesMap.MaxXValue;
  DeltaX:=MaxX-MinX;
  Chart1.Axes.Bottom.SetMinMax(MinX-DeltaX/10,MaxX+DeltaX/10);
  MinY:=SeriesMap.MinYValue;
  MaxY:=SeriesMap.MaxYValue;
  DeltaY:=MaxY-MinY;
  Chart1.Axes.Left.SetMinMax(MinY-DeltaY/10,MaxY+DeltaY/10);
end;

procedure TSecondForm.SpeedButton1Click(Sender: TObject);
begin
  MainForm.Show;
end;

end.
